/////////////////
// main program for dotted-quad calculator
// (see also parser.y)
//
// returns exit_status = 1 if ANY statement in the input
// caused a parse error.
// returns exit_status = 2 if the LAST statement caused a parse error

#include "dq.h"
#include <stdlib.h>	/* for exit() */
#include <stdio.h>	/* for fread() */

int yyparse();

int errflag;
int error_already;

static int Argc;
static char** Argv;
static char** pget;
static char* cget;
static int alldone;

int readit(char * buf, const int max_size) {
  if (max_size <= 0) {
    return 0;
  }
  if (alldone) return 0;
  if (Argc <= 1) {		// no args;  read from stdin
    int c = getchar();
    if (c == EOF) {
      alldone = 1;		// EoF next time
      buf[0] = ';';		// synthesize ";" at end of file
      return 1;
    }
    buf[0] = c;
    return 1;
  } else {			// read args themselves from command line
    if (*cget == 0) {
      pget++;
      if (pget - Argv >= Argc) {
        alldone = 1;
	buf[0] = ';';
	return 1;
      }
      cget = *pget;
      buf[0] = ' ';
    } else {
      buf[0] = *cget++;
    }
    return 1;
  }
}

int main (int argc, char** argv){
  char foo[3];
  Argc = argc;
  Argv = argv;

  pget = argv + 1;
  cget = *pget;
  alldone = 0;
  errflag = 0;
  error_already = 0;

  if (0) {
    for (;;) {
      int rslt;
      rslt = readit(foo, 1);
      if  (!rslt) break;
      putchar(foo[0]);
      putchar('\n');
    }
    exit(0);
  }

  yyparse ();		// should always return 0
  			// since it handles errors itself
  exit(errflag);
}

#include <stdio.h>

void yyerror (char* s){
  if (error_already) {	// message already printed,
    error_already = 0;	// probably by lexer
    return;
  }
  fprintf (stderr, "%s", s);	// typically "parse error"
  if (*Argv) fprintf(stderr, " ... For help, type %s H", *Argv);
  fprintf (stderr, "\n");
}
void help(){
	    fprintf(stdout,
   "A handy calculator for IP addresses, subnet masks, et cetera.\n"
   "Some examples:\n"
   "	  dq 3com.com			## good for numericizing hostnames\n"
   "	192.136.32.249\n"
   "	  dq 3com.com - 13		## arithmetic works; note spacing\n"
   "	192.136.32.236\n"
   "	  dq www.m-w.com		## '-' is part of this name\n"
   "	63.240.197.83\n"
   "	  dq 0-256			## error; looks like hostname; try '0 - 256'\n"
   "	  dq www.xerox.com Hi 13	## high-order bits\n"
   "	204.64.0.0\n"
   "	  dq www.xerox.com Lo 13	## operators must be capitalized\n"
   "	0.0.30.98\n"
   "	  dq 'Hi 24 Xor Lo 15; Swap Last, Last+1' ## Last == result of previous step\n"
   "	255.255.128.255			## ';' separates results by newline\n"
   "	255.128.255.255 255.129.0.0	## ',' separates results by space\n"
   "	  dq -32 / 8, -32 Rsh 3		## Div and Mul are signed; shifts are unsigned\n"
   "	255.255.255.252 31.255.255.252\n"
   "	  echo Card 1.1.1.1 | dq	## read from stdin;  Card counts bits\n"
   "	4\n"
   "	  dq Scalar 255.255.255.0	## Scalar and Card print integers,\n"
   "	-256				##   not dotted quads\n"
   "	  dq 'Scalar 0 -256, Card 0 -256, Card(0 -256)' ## Scalar binds loosely;\n"
   "	-256 255.255.255.0 24		##   other unary operators bind tightly\n"
   "	  dq Not 7, 1.1.1.1 Not 7	## unary Not, binary Not\n"
   "	255.255.255.248 1.1.1.0\n"
   "\n"
"Symbol:  ;  ,  ()  +   -   *   /   %%   &   |   ^   ~   <<  >>\n"
"Word:                      Mul Div Mod And Or  Xor Not Lsh Rsh\n"
"Note:    !     !!  B   UB  B!  B   B   B!  B!  B   UB! B!  B!\n"
"\n"
"Word:    Last Hi  Lo  Swap Card Scalar\n"
"Note:         UB  UB  U    U    U\n"
"where U ==> unary operator, B ==> binary infix operator.\n"
"! ==> symbol means something to the shell, so you have to quote it.\n"
"All Boolean operators are bitwise.\n"
"Written using bison/flex (yacc/lex) on Linux;  should run on any Unix machine\n"
	    );
}
