/* Calculator for Dotted Quads */
/* i.e. for IP addresses (IPv4 only) */

%{
#include <sys/types.h>
#define YYSTYPE int32_t
#include <stdio.h>
#include <math.h>
#include <stdlib.h>		/* for free() */
#include <netinet/in.h>		/* for htonl() */

#include "dq.h"
#include "shift.h"

int yylex(void);

typedef union {
  int32_t i;
  unsigned char b[4];
} fiddle;

YYSTYPE card(YYSTYPE ii){
  YYSTYPE accum = 0;
  while (ii) {
    if (ii & 1) accum++;
    ii = Rsh(ii,1);
  }
  return accum;
}

int32_t last;
int needspace;		// print a space next time

%}


/* BISON Declarations */
%token Snum		/* scalar value */
%token Vnum		/* vector value */
%token VECTOR	/* the Vector keyword */
%token SCALAR	/* the Scalar keyword */
%token LSH		/* left shift */
%token RSH		/* right shift */
%token HI
%token LO
%token CARD
%token LAST
%token SWAP
%token HELP
%token OOPS		/* provoke error , because no rules to handle */

/* precedence rules */
%left WEAK
%left '|'
%left '^'
%left '&' '~'		/* unary ~ same as AND NOT */
%left '-' '+'
%left LSH RSH
%left '*' '/' '%'
%left HI LO
%left UNARY     /*  unary minus, function calls, very tight binding */

/* Grammar follows */
%%
input:	  /* empty string */
	| input line
;

end:	'\n' | ';'
;

line:	  end
	| HELP end 	{help();}
	| error end	{errflag = 2; yyerrok;}
	| scalar end {
	    if (needspace) printf(" ");
	    printf ("%d\n", $1);
	    last = $1;
	    errflag = errflag != 0;
	    needspace = 0;
	  }
	| scalar ',' {
	    if (needspace) printf(" ");
	    printf ("%d", $1);
	    last = $1;
	    errflag = errflag != 0;
	    needspace = 1;
	  }
	| vector end {
	    fiddle f;
	    f.i = htonl($1);	// network byte order is standard
	    if (needspace) printf(" ");
	    printf ("%u.%u.%u.%u\n",
	      (unsigned) f.b[0],	// msb comes first
	      (unsigned) f.b[1],
	      (unsigned) f.b[2],
	      (unsigned) f.b[3]);
	    last = $1;
	    errflag = errflag != 0;
	    needspace = 0;
	  }
	| vector ',' {
	    fiddle f;
	    f.i = htonl($1);
	    if (needspace) printf(" ");
	    printf ("%u.%u.%u.%u",
	      (unsigned) f.b[0],	// msb comes first
	      (unsigned) f.b[1],
	      (unsigned) f.b[2],
	      (unsigned) f.b[3]);
	    last = $1;
	    errflag = errflag != 0;
	    needspace = 1;
	  }
;

item : 	  scalar
	| vector
;

scalar :  SCALAR item  %prec WEAK   { $$ = $2;	 }
	| CARD item  %prec UNARY	{ $$ = card($2);	 }
;

vector:	  Vnum
	| Snum
	| VECTOR item %prec WEAK    { $$ = $2;	 }
	| '-' item  %prec UNARY	{ $$ = -$2;	 }
	| item '+' item		{ $$ = $1 + $3;	}
	| item '-' item		{ $$ = $1 - $3;	 }
	| item '*' item		{ $$ = $1 * $3;	 }
	| item '/' item		{ $$ = $1 / $3;	 }
	| item '%' item		{ $$ = $1 % $3;	 }
	| item '|' item		{ $$ = $1 | $3;	 }
	| item '&' item		{ $$ = $1 & $3;	 }
	| item '~' item		{ $$ = $1 & (-1 ^$3);	 }
	| item LSH item		{ $$ = Lsh($1, $3); }
	| item RSH item		{ $$ = Rsh($1, $3); }
	| item HI item		{ $$ = $1 & (-1 ^Rsh(-1, $3)); }
	| HI item %prec UNARY	{ $$ =      (-1 ^Rsh(-1, $2)); }
	| item LO item		{ $$ = $1 & (-1 ^Lsh(-1, $3)); }
	| LO item %prec UNARY	{ $$ =      (-1 ^Lsh(-1, $2)); }
	| '~' item %prec UNARY	{ $$ = -1 ^ $2;	 }
// ! is a synonym for unary Not:
	| '!' item %prec UNARY	{ $$ = -1 ^ $2;	 }
	| SWAP item %prec UNARY	{
	 $$ = ((($2>>24) & 255)      ) |
	      ((($2>>16) & 255) <<  8) |
	      ((($2>> 8) & 255) << 16) |
	      ((($2    ) & 255) << 24);
	    }
	| item '^' item		{ $$ = $1 ^ $3; }
	| '(' item ')'		{ $$ = $2;	 }
	| LAST			{ $$ = last;	 }
;
%%
